from .server import ProxyServer
import logging


class ProxyDB(ProxyServer):
    url = "http://proxydb.net/?protocol=https&country="

    async def get_proxies(self):
        proxies = []
        try:
            table = self.parser.xpath("//table")[0]
        except Exception as err:
            logging.exception(err)
            return []
        for i in table.xpath("//tbody/tr")[:10]:
            if i.xpath('.//td[5][contains(text(),"HTTPS")]'):
                proxy = str(i.xpath(".//td[1]/a/text()")[0])
                proxies.append(proxy)
        return proxies



from .server import ProxyServer


class FreeProxy(ProxyServer):
    url = "https://free-proxy-list.net/"
    table_attribute: str = 'table table-striped table-bordered'
    table_param: str = 'class'

    async def get_proxies(self):
        proxies = []
        try:
            path = f'//table[@{self.table_param}="{self.table_attribute}"]'
            table = self.parser.xpath(path)[0]
        except IndexError as err:
            print(err)
            return []
        for i in table.xpath("//tbody/tr")[:10]:
            if i.xpath('.//td[7][contains(text(),"yes")]'):
                proxy = ":".join(
                    [i.xpath(".//td[1]/text()")[0], i.xpath(".//td[2]/text()")[0]]
                )
                proxies.append(proxy)
        return proxies



"""Tests.

Testing get_proxies.
"""
import pytest
import asyncio
from datetime import datetime
import timeit
from proxylists import get_proxies, proxy_list, check_address
import pytest_asyncio
from proxylists.proxies import ProxyDB  # getting from proxydb.net


pytestmark = pytest.mark.asyncio


async def test_proxylist(event_loop):
    """ Test Proxy List"""
    error = None
    proxies = []
    try:
        proxies = await proxy_list()
        print(proxies)
    except Exception as err:
        print(err)
        error = err
    pytest.assume(len(proxies) > 0)
    pytest.assume(not error)
    tasks = []
    for proxy in proxies:
        pass  # TODO: checking if is a valid -reachable- proxy


async def test_proxy(event_loop):
    try:
        list = await ProxyDB().get_list()
        print(list)
        for address in list:
            try:
                host, port = address.split(':')
                print('HOST: ', host, port)
                reachable = await check_address(
                    host=host,
                    port=port
                )
                print(host, port, reachable)
                pytest.assume(reachable is True)
                await asyncio.sleep(1)
            except Exception as err:
                print(err)
                pytest.assume(not err)
    finally:
        pytest.assume(len(list) > 0)
